package be.iminds.ilabt.jfed.restapi.debugdata;

import be.iminds.ilabt.jfed.call_log_output.SerializableApiCallDetails;
import be.iminds.ilabt.jfed.call_log_output.SerializableApiCallDetailsFactory;
import be.iminds.ilabt.jfed.log.ApiCallDetails;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.TextUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

public class ClientApiCallDDBuilder {
    private static final Logger LOG = LoggerFactory.getLogger(ClientApiCallDDBuilder.class);

    private String api;
    private String apiVersion;
    private Timestamp start;
    private Timestamp stop;
    private ClientApiCallDD.SentData sent;
    private ClientApiCallDD.ReceivedData received;
    private ClientApiCallDD.ConnectionDD connection;
    private String stacktrace;

    public ClientApiCallDDBuilder() {
    }

    public ClientApiCallDDBuilder(ClientApiCallDD orig) {
        this.api = orig.getApi();
        this.apiVersion = orig.getApiVersion();
        this.start = orig.getStart();
        this.stop = orig.getStop();
        this.sent = orig.getSent();
        this.received = orig.getReceived();
        this.stacktrace = orig.getStacktrace();
        this.connection = orig.getConnection();
    }

    public ClientApiCallDDBuilder(SerializableApiCallDetails apiCallDetails) {
        this.api = apiCallDetails.getApiName();
        this.apiVersion = "TODO: support in apiCallDetails";
        this.start = apiCallDetails.getStartTime() == null ?
                null :
                new Timestamp(apiCallDetails.getStartTime().getTime());
        this.stop = apiCallDetails.getStopTime() == null ?
                null :
                new Timestamp(apiCallDetails.getStopTime().getTime());
        this.stacktrace = apiCallDetails.getExceptionString();

        if (apiCallDetails.getHttpResponse() == null && apiCallDetails.getHttpResponseHeaders() == null && apiCallDetails.getXmlRpcResponseJsonString() == null) {
            this.received = null;
        } else {
            this.received = new ClientApiCallDD.ReceivedData(
                    apiCallDetails.getHttpResponseStatusLine(),
                    apiCallDetails.getHttpResponseHeaders(),
                    apiCallDetails.getHttpResponse(),
                    apiCallDetails.getXmlRpcResponseJsonObject()
            );
        }

        if (apiCallDetails.getHttpRequest() == null && apiCallDetails.getHttpRequestHeaders() == null &&
                apiCallDetails.getHttpRequestStatusLine() == null && apiCallDetails.getXmlRpcRequestJsonString() == null) {
            this.sent = null;
        } else {
            this.sent = new ClientApiCallDD.SentData(
                    apiCallDetails.getHttpRequestStatusLine(),
                    apiCallDetails.getHttpRequestHeaders(),
                    apiCallDetails.getHttpRequest(),
                    apiCallDetails.getGeniMethodName(),
                    apiCallDetails.getXmlRpcRequestJsonObject()
            );
        }

        if (apiCallDetails.getCallServerUrl() != null) {
            this.connection = null;
        } else {
            try {
                this.connection = new ClientApiCallDD.ConnectionDD(
                        apiCallDetails.getConnectionAuthenticationMethod(),
                        apiCallDetails.getConnectionProtocol(),
                        apiCallDetails.getConnectionType(),
                        new URI(apiCallDetails.getCallServerUrl()),
                        null/*server*/,
                        GeniUrn.parse(apiCallDetails.getConnectionSslAuthUserUrn())
                );
            } catch (IllegalArgumentException e) {
                LOG.error("Bad connection Auth, Protocol or Type in SerializableApiCallDetails", e);
            } catch (URISyntaxException e) {
                LOG.error("Bad URI in SerializableApiCallDetails", e);
            }
        }
    }

    public ClientApiCallDDBuilder(ApiCallDetails apiCallDetails) {
        this(new SerializableApiCallDetailsFactory().createFromApiCallDetails(apiCallDetails));

        //override ConnectionDD because Server cannot be found in SerializableApiCallDetails but it is in ApiCallDetails
        if (apiCallDetails.getConnectionConfig() != null) {
            try {
                this.connection = new ClientApiCallDD.ConnectionDD(
                        apiCallDetails.getConnectionConfig().getAuthentication(),
                        apiCallDetails.getConnectionConfig().getProtocol(),
                        apiCallDetails.getConnectionConfig().getType(),
                        apiCallDetails.getConnectionConfig().getServerUrl() == null ?
                                null :
                                apiCallDetails.getConnectionConfig().getServerUrl().toURI(),
                        apiCallDetails.getServer(),
                        apiCallDetails.getConnectionConfig().getDebugInfo() == null ||
                                apiCallDetails.getConnectionConfig().getDebugInfo().getUser() == null ?
                                null :
                                apiCallDetails.getConnectionConfig().getDebugInfo().getUser().getUserUrn()
                );
            } catch (URISyntaxException e) {
                LOG.error("URL in ApiCallDetails is not a URI", e);
            }
        }
    }

    public ClientApiCallDDBuilder setApi(String api) {
        this.api = api;
        return this;
    }

    public ClientApiCallDDBuilder setApiVersion(String apiVersion) {
        this.apiVersion = apiVersion;
        return this;
    }

    public ClientApiCallDDBuilder setStart(Timestamp start) {
        this.start = start;
        return this;
    }

    public ClientApiCallDDBuilder setStop(Timestamp stop) {
        this.stop = stop;
        return this;
    }

    public ClientApiCallDDBuilder setSent(ClientApiCallDD.SentData sent) {
        this.sent = sent;
        return this;
    }

    public ClientApiCallDDBuilder setReceived(ClientApiCallDD.ReceivedData received) {
        this.received = received;
        return this;
    }

    public ClientApiCallDDBuilder setStacktrace(String stacktrace) {
        this.stacktrace = stacktrace;
        return this;
    }

    public ClientApiCallDD create() {
        return new ClientApiCallDD(api, apiVersion, start, stop, sent, received, stacktrace, connection);
    }

    public static List<ClientApiCallDD> createAll(@Nullable List<ClientApiCallDDBuilder> builders) {
        if (builders == null)
            return null;

        List<ClientApiCallDD> res = new ArrayList<>();
        for (ClientApiCallDDBuilder b : builders) {
            res.add(b.create());
        }
        return res;
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(create());
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}