package be.iminds.ilabt.jfed.restapi;

import be.iminds.ilabt.jfed.util.IOUtils;
import io.dropwizard.testing.junit.DropwizardAppRule;
import liquibase.Liquibase;
import liquibase.database.Database;
import liquibase.database.DatabaseFactory;
import liquibase.database.jvm.JdbcConnection;
import liquibase.exception.LiquibaseException;
import liquibase.resource.ClassLoaderResourceAccessor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

/**
 * TestPostgresDB
 */
public class TestDB {
    private static final Logger LOG = LoggerFactory.getLogger(TestDB.class);

    private final DropwizardAppRule<JFedRestApiConfiguration> rule;
    private String clearScript;
    private String fillScript;

    public TestDB(DropwizardAppRule<JFedRestApiConfiguration> rule) {
        this.rule = rule;
        assert rule != null;
        try {
            clearScript = IOUtils.resourceToString("clear_tables.sql", TestDB.class);
            fillScript = IOUtils.resourceToString("fill_tables.sql", TestDB.class);
        } catch (Exception e) {
            throw new RuntimeException("Exception while loading DB scrips", e);
        }
    }

    private Connection setupConnection() {
        try {
            //delete and recreate test DB
            Properties connectionProps = new Properties();
            connectionProps.put("user", rule.getConfiguration().getDataSourceFactory().getUser());
            connectionProps.put("password", rule.getConfiguration().getDataSourceFactory().getPassword());

            Connection con = DriverManager.getConnection(
                    rule.getConfiguration().getDataSourceFactory().getUrl(),
                    connectionProps);
            assert con != null;
            return con;
        } catch (Exception e) {
            throw new RuntimeException("Exception while setting up test DB helper", e);
        }
    }

    public void clearAllTables() throws SQLException {
        Connection con = setupConnection();
        try {
            Statement stmt = con.createStatement();
            stmt.execute(clearScript);
            stmt.close();
        } finally {
            con.close();
        }
    }
    public void dropAllTables() throws SQLException {
        Connection con = setupConnection();
        Database database = null;
        try {
            database = DatabaseFactory.getInstance().findCorrectDatabaseImplementation(new JdbcConnection(con));
            Liquibase liquibase = new Liquibase("migrations.xml", new ClassLoaderResourceAccessor(), database);
            liquibase.dropAll();
        } catch (LiquibaseException e) {
            LOG.error("Liquibase failed", e);
            throw new RuntimeException("Liquibase failed", e);
        } finally {
            con.close();
        }
    }
    public void recreateTableStructure() throws SQLException {
        dropAllTables();
        syncTableStructure();
    }
    public void syncTableStructure() throws SQLException {
        Connection con = setupConnection();
        Database database = null;
        try {
            database = DatabaseFactory.getInstance().findCorrectDatabaseImplementation(new JdbcConnection(con));
            Liquibase liquibase = new Liquibase("migrations.xml", new ClassLoaderResourceAccessor(), database);
            liquibase.update("");
        } catch (LiquibaseException e) {
            LOG.error("Liquibase failed", e);
            throw new RuntimeException("Liquibase failed", e);
        } finally {
            con.close();
        }
    }
    public void fillAllTables() throws SQLException {
        Connection con = setupConnection();
        try {
            Statement stmt = con.createStatement();
            stmt.execute(fillScript);
            stmt.close();
        } finally {
            con.close();
        }
    }

    public void executeStatement(String statementString) throws SQLException {
        Connection con = setupConnection();
        try {
            Statement stmt = con.createStatement();
            stmt.execute(statementString);
            stmt.close();
        } finally {
            con.close();
        }
    }

    /**
     * Clear all tables, leaving DB structure as is.
     * Then fill all tables with initial values.
     * */
    public void refillDB() throws SQLException {
        clearAllTables();
        fillAllTables();
    }
}
